@@ -14,7 +14,7 @@ mem_alloc(size_t size)
 
    mem = calloc(1, size);
    if (mem) return mem;
-   ERR("%s: Error. %s:%i memory allocation of %zi bytes failed. %s",
+   ERR("%s: Error. %s:%i memory allocation of %zu bytes failed. %s",
        progname, file_in, line, size, strerror(errno));
    exit(-1);
    return NULL;
@@ -27,7 +27,7 @@ mem_strdup(const char *s)
 
    str = strdup(s);
    if (str) return str;
-   ERR("%s: Error. %s:%i memory allocation of %zi bytes failed. %s. string being duplicated: \"%s\"",
+   ERR("%s: Error. %s:%i memory allocation of %zu bytes failed. %s. string being duplicated: \"%s\"",
        progname, file_in, line, strlen(s) + 1, strerror(errno), s);
    exit(-1);
    return NULL;
