@@ -33,6 +33,12 @@ void *alloca (size_t);
 #include <Ecore.h>
 #include <Ecore_File.h>
 
+#ifdef _WIN32
+# define EPP_EXT ".exe"
+#else
+# define EPP_EXT
+#endif
+
 static void  new_object(void);
 static void  new_statement(void);
 static char *perform_math (char *input);
@@ -77,6 +83,42 @@ static int   verbatim_line1 = 0;
 static int   verbatim_line2 = 0;
 static char *verbatim_str = NULL;
 
+static void
+err_show_stack(void)
+{
+   char *s;
+   
+   s = stack_id();
+   if (s)
+     {
+        printf("PARSE STACK:\n%s\n", s);
+        free(s);
+     }
+   else
+      printf("NO PARSE STACK\n");
+}
+
+static void
+err_show_params(void)
+{
+   Eina_List *l;
+   char *p;
+
+   printf("PARAMS:");
+   EINA_LIST_FOREACH(params, l, p)
+     {
+        printf(" %s", p);
+     }
+   printf("\n");
+}
+
+static void
+err_show(void)
+{
+   err_show_stack();
+   err_show_params();
+}
+
 static void
 new_object(void)
 {
@@ -113,6 +155,7 @@ new_object(void)
 	ERR("%s: Error. %s:%i unhandled keyword %s",
 	    progname, file_in, line - 1,
 	    (char *)eina_list_data_get(eina_list_last(stack)));
+        err_show();
 	exit(-1);
      }
    free(id);
@@ -143,6 +186,7 @@ new_statement(void)
 	ERR("%s: Error. %s:%i unhandled keyword %s",
 	    progname, file_in, line - 1,
 	    (char *)eina_list_data_get(eina_list_last(stack)));
+        err_show();
 	exit(-1);
      }
    free(id);
@@ -194,8 +238,6 @@ next_token(char *p, char *end, char **new_p, int *delim)
    int in_comment_sa  = 0;
    int had_quote = 0;
    int is_escaped = 0;
-   char *cpp_token_line = NULL;
-   char *cpp_token_file = NULL;
 
    *delim = 0;
    if (p >= end) return NULL;
@@ -205,8 +247,6 @@ next_token(char *p, char *end, char **new_p, int *delim)
 	  {
 	     in_comment_ss = 0;
 	     in_comment_cpp = 0;
-	     cpp_token_line = NULL;
-	     cpp_token_file = NULL;
 	     line++;
 	  }
 	if ((!in_comment_ss) && (!in_comment_sa))
@@ -229,10 +269,8 @@ next_token(char *p, char *end, char **new_p, int *delim)
 
 	     /* handle cpp comments */
 	     /* their line format is
-	      * # <line no. of next line> <filename from next line on> [??]
+	      * #line <line no. of next line> <filename from next line on> [??]
 	      */
-	     cpp_token_line = NULL;
-	     cpp_token_file = NULL;
 
 	     pp = p;
 	     while ((pp < end) && (*pp != '\n'))
@@ -426,6 +464,7 @@ stack_chop_top(void)
      {
 	ERR("%s: Error. parse error %s:%i. } marker without matching { marker",
 	    progname, file_in, line - 1);
+        err_show();
 	exit(-1);
      }
 }
@@ -454,6 +493,7 @@ parse(char *data, off_t size)
 	  {
 	     ERR("%s: Error. parse error %s:%i. %c marker before ; marker",
 		 progname, file_in, line - 1, *token);
+             err_show();
 	     exit(-1);
 	  }
 	else if (delim)
@@ -465,6 +505,7 @@ parse(char *data, off_t size)
 		    {
 		       ERR("%s: Error. parse error %s:%i. } marker before ; marker",
 			       progname, file_in, line - 1);
+                       err_show();
 		       exit(-1);
 		    }
 		  else
@@ -492,6 +533,7 @@ parse(char *data, off_t size)
 		    {
 		       ERR("%s: Error. parse error %s:%i. { marker before ; marker",
 			   progname, file_in, line - 1);
+                       err_show();
 		       exit(-1);
 		    }
 	       }
@@ -575,6 +617,7 @@ parse(char *data, off_t size)
 			 {
 			    ERR("%s: Error. parse error %s:%i. { marker does not have matching } marker",
 				progname, file_in, line - 1);
+                            err_show();
 			    exit(-1);
 			 }
 		       new_object();
@@ -638,7 +681,7 @@ get_verbatim_line2(void)
 void
 compile(void)
 {
-   char buf[4096];
+   char buf[4096], buf2[4096];
    char inc[4096];
    static char tmpn[4096];
    int fd;
@@ -659,7 +702,7 @@ compile(void)
    p = strrchr(inc, '/');
    if (!p) strcpy(inc, "./");
    else *p = 0;
-   snprintf (tmpn, PATH_MAX, "%s/edje_cc.edc-tmp-XXXXXX", tmp_dir);
+   snprintf(tmpn, PATH_MAX, "%s/edje_cc.edc-tmp-XXXXXX", tmp_dir);
    fd = mkstemp(tmpn);
    if (fd >= 0)
      {
@@ -673,18 +716,17 @@ compile(void)
 	  def = mem_strdup("");
 	else
 	  {
-	     Eina_List *l;
 	     int len;
-	     char *data;
+	     char *define;
 
 	     len = 0;
-	     EINA_LIST_FOREACH(defines, l, data)
-	       len += strlen(data) + 1;
+	     EINA_LIST_FOREACH(defines, l, define)
+	       len += strlen(define) + 1;
 	     def = mem_alloc(len + 1);
 	     def[0] = 0;
-	     EINA_LIST_FOREACH(defines, l, data)
+	     EINA_LIST_FOREACH(defines, l, define)
 	       {
-		  strcat(def, data);
+		  strcat(def, define);
 		  strcat(def, " ");
 	       }
 	  }
@@ -692,69 +734,28 @@ compile(void)
 	/*
 	 * Run the input through the C pre-processor.
 	 */
-
-	/*
-	 * On OpenSolaris, the default cpp is located in different places.
-	 * Alan Coppersmith told me to do what xorg does: using /usr/ccs/lib/cpp
-	 *
-	 * Also, that preprocessor is not managing C++ comments, so pass the
-	 * sun cc preprocessor just after.
-	 */
         ret = -1;
-        if (ecore_file_exists("/usr/ccs/lib/cpp"))
+        snprintf(buf2, sizeof(buf2), "%s/edje/utils/epp" EPP_EXT, 
+                 eina_prefix_lib_get(pfx));
+        if (ecore_file_exists(buf2))
           {
-             snprintf(buf, sizeof(buf), "/usr/ccs/lib/cpp -I%s %s %s %s",
-                      inc, def, file_in, tmpn);
+             snprintf(buf, sizeof(buf), "%s %s -I%s %s -o %s",
+                      buf2, file_in, inc, def, tmpn);
              ret = system(buf);
-             if (ret == 0)
-               {
-                  static char tmpn2[4096];
-                  
-                  snprintf (tmpn2, PATH_MAX, "%s/edje_cc.edc-tmp-XXXXXX", tmp_dir);
-                  fd = mkstemp(tmpn2);
-                  if (fd >= 0)
-                    {
-                       close(fd); 
-                       snprintf (buf, 4096, "cc -E -I%s %s -o %s %s",
-                                 inc, def, tmpn2, tmpn);
-                       ret = system(buf);
-                       snprintf(tmpn, 4096, "%s", tmpn2);
-                    }
-               }
           }
-
-        /* Trying gcc and other syntax */
-	if (ret != 0)
-	  {
-	     snprintf(buf, sizeof(buf), "%s -I%s %s -E -o %s -std=c99 - < %s",
-                      getenv("CC") ? getenv("CC") : "cc",
-		      inc, def, tmpn, file_in);
-	     ret = system(buf);
-	  }
-        /* Trying suncc syntax */
-	if (ret != 0)
-	  {
-	     snprintf(buf, sizeof(buf), "%s -I%s %s -E -o %s -xc99 - < %s",
-                      getenv("CC") ? getenv("CC") : "cc",
-		      inc, def, tmpn, file_in);
-	     ret = system(buf);
-	  }
+        else
+          {
+             ERR("Error. Cannot run epp: %s", buf2);
+             exit(-1);
+          }
 	if (ret == EXIT_SUCCESS)
 	  file_in = tmpn;
+        else
+          {
+             ERR("Error. Exit code of epp not clean: %i", ret);
+             exit(-1);
+          }
 	free(def);
-/* OLD CODE
-	snprintf(buf, sizeof(buf), "cat %s | cpp -I%s %s -E -o %s",
-		 file_in, inc, def, tmpn);
-	ret = system(buf);
-	if (ret < 0)
-	  {
-	     snprintf(buf, sizeof(buf), "gcc -I%s %s -E -o %s %s",
-		      inc, def, tmpn, file_in);
-	     ret = system(buf);
-	  }
-	if (ret >= 0) file_in = tmpn;
-	free(def);
- */
      }
    fd = open(file_in, O_RDONLY | O_BINARY, S_IRUSR | S_IWUSR);
    if (fd < 0)
@@ -765,15 +766,14 @@ compile(void)
      }
    if (verbose)
      {
-	INF("%s: Opening \"%s\" for input",
-	       progname, file_in);
+	INF("%s: Opening \"%s\" for input", progname, file_in);
      }
 
    size = lseek(fd, 0, SEEK_END);
    lseek(fd, 0, SEEK_SET);
    data = malloc(size);
    if (data && (read(fd, data, size) == size))
-	parse(data, size);
+      parse(data, size);
    else
      {
 	ERR("%s: Error. cannot read file \"%s\". %s",
@@ -784,11 +784,13 @@ compile(void)
    close(fd);
 
    EINA_LIST_FOREACH(edje_file->styles, l, stl)
-      if (!stl->name)
-	{
-	   ERR("%s: Error. style must have a name.", progname);
-	   exit(-1);
-	}
+     {
+        if (!stl->name)
+          {
+             ERR("%s: Error. style must have a name.", progname);
+             exit(-1);
+          }
+     }
 }
 
 int
@@ -813,6 +815,7 @@ is_num(int n)
      {
 	ERR("%s: Error. %s:%i no parameter supplied as argument %i",
 		progname, file_in, line - 1, n + 1);
+        err_show();
 	exit(-1);
      }
    if (str[0] == 0) return 0;
@@ -837,6 +840,7 @@ parse_str(int n)
      {
 	ERR("%s: Error. %s:%i no parameter supplied as argument %i",
 	    progname, file_in, line - 1, n + 1);
+        err_show();
 	exit(-1);
      }
    s = mem_strdup(str);
@@ -872,6 +876,7 @@ _parse_enum(char *str, va_list va)
 	     fprintf(stderr, "\n");
 	     va_end(va2);
 	     va_end(va);
+             err_show();
 	     exit(-1);
 	  }
 
@@ -900,6 +905,7 @@ parse_enum(int n, ...)
      {
 	ERR("%s: Error. %s:%i no parameter supplied as argument %i",
 		progname, file_in, line - 1, n + 1);
+        err_show();
 	exit(-1);
      }
 
@@ -937,6 +943,7 @@ parse_int(int n)
      {
 	ERR("%s: Error. %s:%i no parameter supplied as argument %i",
 	    progname, file_in, line - 1, n + 1);
+        err_show();
 	exit(-1);
      }
    i = my_atoi(str);
@@ -954,6 +961,7 @@ parse_int_range(int n, int f, int t)
      {
 	ERR("%s: Error. %s:%i no parameter supplied as argument %i",
 	    progname, file_in, line - 1, n + 1);
+        err_show();
 	exit(-1);
      }
    i = my_atoi(str);
@@ -961,6 +969,7 @@ parse_int_range(int n, int f, int t)
      {
 	ERR("%s: Error. %s:%i integer %i out of range of %i to %i inclusive",
 	    progname, file_in, line - 1, i, f, t);
+        err_show();
 	exit(-1);
      }
    return i;
@@ -977,6 +986,7 @@ parse_bool(int n)
      {
 	ERR("%s: Error. %s:%i no parameter supplied as argument %i",
 	    progname, file_in, line - 1, n + 1);
+        err_show();
 	exit(-1);
      }
 
@@ -997,6 +1007,7 @@ parse_bool(int n)
      {
 	ERR("%s: Error. %s:%i integer %i out of range of 0 to 1 inclusive",
 	    progname, file_in, line - 1, i);
+        err_show();
 	exit(-1);
      }
    return i;
@@ -1013,6 +1024,7 @@ parse_float(int n)
      {
 	ERR("%s: Error. %s:%i no parameter supplied as argument %i",
 	    progname, file_in, line - 1, n + 1);
+        err_show();
 	exit(-1);
      }
    i = my_atof(str);
@@ -1030,6 +1042,7 @@ parse_float_range(int n, double f, double t)
      {
 	ERR("%s: Error. %s:%i no parameter supplied as argument %i",
 	    progname, file_in, line - 1, n + 1);
+        err_show();
 	exit(-1);
      }
    i = my_atof(str);
@@ -1037,11 +1050,18 @@ parse_float_range(int n, double f, double t)
      {
 	ERR("%s: Error. %s:%i float %3.3f out of range of %3.3f to %3.3f inclusive",
 	    progname, file_in, line - 1, i, f, t);
+        err_show();
 	exit(-1);
      }
    return i;
 }
 
+int
+get_arg_count(void)
+{
+   return eina_list_count (params);
+}
+
 void
 check_arg_count(int required_args)
 {
@@ -1049,8 +1069,9 @@ check_arg_count(int required_args)
 
    if (num_args != required_args)
      {
-       ERR("%s: Error. %s:%i got %i arguments, but expected %i",
-	   progname, file_in, line - 1, num_args, required_args);
+        ERR("%s: Error. %s:%i got %i arguments, but expected %i",
+            progname, file_in, line - 1, num_args, required_args);
+        err_show();
 	exit(-1);
      }
 }
@@ -1065,6 +1086,7 @@ check_min_arg_count(int min_required_args)
 	ERR("%s: Error. %s:%i got %i arguments, "
 		"but expected at least %i",
 	    progname, file_in, line - 1, num_args, min_required_args);
+        err_show();
 	exit(-1);
      }
 }
