@@ -1,5 +1,5 @@
 #include "edje_private.h"
-
+#define _ELLIP_STR "\xE2\x80\xA6"
 
 /* returns with and height for this part.
  *
@@ -60,6 +60,7 @@ _edje_text_part_on_del(Edje *ed, Edje_Part *pt)
    Edje_Part_Description_Text *desc;
    unsigned int i;
 
+   if (!pt) return;
    if (pt->type != EDJE_PART_TYPE_TEXT
        && pt->type != EDJE_PART_TYPE_TEXTBLOCK)
      return ;
@@ -86,12 +87,12 @@ _edje_text_fit_set(char *buf, const char *text, int c1, int c2)
 
    if (c1 >= 0)
      {
-	strcpy(buf, "...");
+	strcpy(buf, _ELLIP_STR);
 
 	if (c2 >= 0)
 	  {
 	     strncat(buf, text + c1, c2 - c1);
-	     strcat(buf, "...");
+	     strcat(buf, _ELLIP_STR);
 	  }
 	else
 	  strcat(buf, text + c1);
@@ -102,7 +103,7 @@ _edje_text_fit_set(char *buf, const char *text, int c1, int c2)
 	  {
 	     strncpy(buf, text, c2);
 	     buf[c2] = 0;
-	     strcat(buf, "...");
+	     strcat(buf, _ELLIP_STR);
 	  }
 	else
 	  strcpy(buf, text);
@@ -147,9 +148,14 @@ _edje_text_fit_x(Edje *ed, Edje_Real_Part *ep,
           uc1 = evas_object_text_last_up_to_pos(ep->object,
                 -p + l, th / 2);
 	if (params->type.text.elipsis != 1.0)
-          /* should be the last in text! not the rightmost */
-          uc2 = evas_object_text_last_up_to_pos(ep->object,
-                -p + sw - r, th / 2);
+          {
+             /* should be the last in text! not the rightmost */
+             if ((-p + sw -r) < 0)
+                uc2 = evas_object_text_last_up_to_pos(ep->object, 0, th / 2);
+             else
+                uc2 = evas_object_text_last_up_to_pos(ep->object,
+                      -p + sw - r, th / 2);
+          }
 	if ((uc1 < 0) && (uc2 < 0))
 	  {
 	     uc1 = 0;
@@ -176,7 +182,7 @@ _edje_text_fit_x(Edje *ed, Edje_Real_Part *ep,
 
    /* Convert uc1, uc2 -> c1, c2 */
    i = 0;
-   if (uc1 > 0)
+   if (uc1 >= 0)
      {
         c1 = 0;
         for ( ; i < uc1 ; i++)
@@ -184,15 +190,24 @@ _edje_text_fit_x(Edje *ed, Edje_Real_Part *ep,
              c1 = evas_string_char_next_get(text, c1, NULL);
           }
      }
-   if (uc2 > 0)
+   if (uc2 >= 0)
      {
-        c2 = c1;
+        if (c1 >= 0)
+          {
+             c2 = c1;
+          }
+        else
+          {
+             c2 = 0;
+          }
         for ( ; i < uc2 ; i++)
           {
              c2 = evas_string_char_next_get(text, c2, NULL);
           }
      }
 
+   buf[0] = '\0';
+
    while (((c1 >= 0) || (c2 >= 0)) && (tw > sw))
      {
 	loop++;
@@ -321,7 +336,7 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 			Edje_Calc_Params *params,
 			Edje_Part_Description_Text *chosen_desc)
 {
-   const char	*text, *str;
+   const char	*text = NULL;
    const char	*font;
    char		*font2 = NULL;
    char         *sfont = NULL;
@@ -329,6 +344,7 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
    Evas_Coord	 tw, th;
    Evas_Coord	 sw, sh;
    int		 inlined_font = 0, free_text = 0;
+   Eina_Bool     same_text = EINA_FALSE;
    FLOAT_T       sc;
 
    sc = ed->scale;
@@ -336,7 +352,7 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
    text = edje_string_get(&chosen_desc->text.text);
    font = _edje_text_class_font_get(ed, chosen_desc, &size, &sfont);
 
-   if (ep->text.text) text = (char *) ep->text.text;
+   if (ep->text.text) text = ep->text.text;
    if (ep->text.font) font = ep->text.font;
    if (ep->text.size > 0) size = ep->text.size;
 
@@ -363,13 +379,12 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 
 	if (fnt)
 	  {
-             char *font2;
-             
              size_t len = strlen(font) + sizeof("edje/fonts/") + 1;
              font2 = alloca(len);
              sprintf(font2, "edje/fonts/%s", font);
              font = font2;
 	     inlined_font = 1;
+             font2 = NULL;
 	  }
      }
 
@@ -393,28 +408,35 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
      }
 
    size = params->type.text.size;
+   if (!text) text = "";
+
+   if ((text == ep->text.cache.in_str)
+       || (text && ep->text.cache.in_str && !strcmp(ep->text.cache.in_str, text)))
+     {
+        text = ep->text.cache.in_str;
+        same_text = EINA_TRUE;
+     }
 
    if ((ep->text.cache.in_size == size) &&
        (ep->text.cache.in_w == sw) &&
        (ep->text.cache.in_h == sh) &&
        (ep->text.cache.in_str) &&
-       (text) &&
-       (!strcmp(ep->text.cache.in_str, text)) &&
+       same_text &&
        (ep->text.cache.align_x == params->type.text.align.x) &&
        (ep->text.cache.align_y == params->type.text.align.y) &&
        (ep->text.cache.elipsis == params->type.text.elipsis) &&
        (ep->text.cache.fit_x == chosen_desc->text.fit_x) &&
        (ep->text.cache.fit_y == chosen_desc->text.fit_y))
      {
-	text = (char *)ep->text.cache.out_str;
+	text = ep->text.cache.out_str;
 	size = ep->text.cache.out_size;
 
-	if (!text) text = "";
-
 	goto arrange_text;
      }
-   if (ep->text.cache.in_str) eina_stringshare_del(ep->text.cache.in_str);
-   ep->text.cache.in_str = eina_stringshare_add(text);
+   if (!same_text)
+     {
+        eina_stringshare_replace(&ep->text.cache.in_str, text);
+     }
    ep->text.cache.in_size = size;
    if (chosen_desc->text.fit_x && (ep->text.cache.in_str && eina_stringshare_strlen(ep->text.cache.in_str) > 0))
      {
@@ -425,39 +447,22 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 	evas_object_text_font_set(ep->object, font, size);
 	evas_object_text_text_set(ep->object, text);
 	part_get_geometry(ep, &tw, &th);
-	if (tw > sw)
-	  {
-	     while ((tw > sw) && (size > 0) && (tw != 0))
-	       {
-		  int psize = size;
-		  size = (size * sw) / tw;
-		  if ((psize - size) <= 0) size = psize - 1;
-		  if (inlined_font) evas_object_text_font_source_set(ep->object, ed->path);
-		  else evas_object_text_font_source_set(ep->object, NULL);
+        /* Find the wanted font size */
+	if ((tw != sw) && (size > 0) && (tw != 0))
+          {
+             size = (size * sw) / tw;
 
-		  if (ep->part->scale) evas_object_scale_set(ep->object, TO_DOUBLE(sc));
-		  evas_object_text_font_set(ep->object, font, size);
-		  part_get_geometry(ep, &tw, &th);
-		  if ((size > 0) && (tw == 0)) break;
-	       }
-	  }
-	else if (tw < sw)
-	  {
-	     while ((tw < sw) && (size > 0) && (tw != 0))
-	       {
-		  int psize = size;
-		  size = (size * sw) / tw;
-		  /* fprintf(stderr, "size = %i (%i, %i)\n", size, sw, tw); */
-		  if ((psize - size) >= 0) size = psize + 1;
-		  if (inlined_font) evas_object_text_font_source_set(ep->object, ed->path);
-		  else evas_object_text_font_source_set(ep->object, NULL);
+             if (inlined_font) evas_object_text_font_source_set(ep->object, ed->path);
+             else evas_object_text_font_source_set(ep->object, NULL);
 
-		  if (ep->part->scale) evas_object_scale_set(ep->object, TO_DOUBLE(sc));
-		  evas_object_text_font_set(ep->object, font, size);
-		  part_get_geometry(ep, &tw, &th);
-		  if ((size > 0) && (tw == 0)) break;
-	       }
+             if (ep->part->scale) evas_object_scale_set(ep->object, TO_DOUBLE(sc));
+             evas_object_text_font_set(ep->object, font, size);
+             part_get_geometry(ep, &tw, &th);
 	  }
+
+        /* FIXME: This should possibly be replaced by more proper handling,
+         * but it's still way better than what was here before. */
+        if (tw > sw) size--;
      }
    if (chosen_desc->text.fit_y && (ep->text.cache.in_str && eina_stringshare_strlen(ep->text.cache.in_str) > 0))
      {
@@ -543,9 +548,18 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 	     size = current - 1;
 	  }
      }
-   if (size < 1) size = 1;
 
-   if (!chosen_desc->text.fit_x)
+   /* Make sure the size is in range */
+   if (size < 1)
+      size = 1;
+   else if ((size > chosen_desc->text.size_range_max) &&
+            (chosen_desc->text.size_range_max > 0))
+      size = chosen_desc->text.size_range_max;
+   else if (size < chosen_desc->text.size_range_min)
+      size = chosen_desc->text.size_range_min;
+
+   /* Handle ellipsis */
+   if (!chosen_desc->text.min_x)
      {
 	if (inlined_font) evas_object_text_font_source_set(ep->object, ed->path);
 	else evas_object_text_font_source_set(ep->object, NULL);
@@ -553,9 +567,7 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 	text = _edje_text_fit_x(ed, ep, params, text, font, size, sw, &free_text);
      }
 
-   str = eina_stringshare_add(text);
-   if (ep->text.cache.out_str) eina_stringshare_del(ep->text.cache.out_str);
-   ep->text.cache.out_str = str;
+   eina_stringshare_replace(&ep->text.cache.out_str, text);
    ep->text.cache.in_w = sw;
    ep->text.cache.in_h = sh;
    ep->text.cache.out_size = size;
@@ -573,8 +585,28 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
    evas_object_text_font_set(ep->object, font, size);
    evas_object_text_text_set(ep->object, text);
    part_get_geometry(ep, &tw, &th);
-   ep->text.offset.x = TO_INT(SCALE(params->type.text.align.x, (sw - tw)));
-   ep->text.offset.y = TO_INT(SCALE(params->type.text.align.y, (sh - th)));
+   /* Handle alignment */
+     {
+        double align_x;
+        if (params->type.text.align.x < 0.0)
+          {
+             if (evas_object_text_direction_get(ep->object) ==
+                   EVAS_BIDI_DIRECTION_RTL)
+               {
+                  align_x = 1.0;
+               }
+             else
+               {
+                  align_x = 0.0;
+               }
+          }
+        else
+          {
+             align_x = params->type.text.align.x;
+          }
+        ep->text.offset.x = TO_INT(SCALE(align_x, (sw - tw)));
+        ep->text.offset.y = TO_INT(SCALE(params->type.text.align.y, (sh - th)));
+     }
 
    evas_object_move(ep->object,
 		    ed->x + params->x + ep->text.offset.x,
@@ -584,6 +616,7 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
    else evas_object_hide(ep->object);
      {
 	Evas_Text_Style_Type style;
+        Edje_Text_Effect effect;
 
 	style = EVAS_TEXT_STYLE_PLAIN;
 
@@ -592,50 +625,46 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 			      (params->color.g * params->color.a) / 255,
 			      (params->color.b * params->color.a) / 255,
 			      params->color.a);
-
-	if ((ep->part->effect == EDJE_TEXT_EFFECT_NONE) ||
-	      (ep->part->effect == EDJE_TEXT_EFFECT_PLAIN))
-	  {
+        effect = ep->part->effect;
+        switch (effect & EDJE_TEXT_EFFECT_MASK_BASIC)
+          {
+           case EDJE_TEXT_EFFECT_NONE:
+           case EDJE_TEXT_EFFECT_PLAIN:
 	     style = EVAS_TEXT_STYLE_PLAIN;
-	  }
-	else if (ep->part->effect == EDJE_TEXT_EFFECT_OUTLINE)
-	  {
+             break;
+           case EDJE_TEXT_EFFECT_OUTLINE:
 	     style = EVAS_TEXT_STYLE_OUTLINE;
 	     evas_object_text_outline_color_set(ep->object,
 					        (params->type.text.color2.r * params->type.text.color2.a) / 255,
 					        (params->type.text.color2.g * params->type.text.color2.a) / 255,
 					        (params->type.text.color2.b * params->type.text.color2.a) / 255,
 						params->type.text.color2.a);
-	  }
-	else if (ep->part->effect == EDJE_TEXT_EFFECT_SOFT_OUTLINE)
-	  {
+             break;
+           case EDJE_TEXT_EFFECT_SOFT_OUTLINE:
 	     style = EVAS_TEXT_STYLE_SOFT_OUTLINE;
 	     evas_object_text_outline_color_set(ep->object,
 						(params->type.text.color2.r * params->type.text.color2.a) / 255,
 						(params->type.text.color2.g * params->type.text.color2.a) / 255,
 						(params->type.text.color2.b * params->type.text.color2.a) / 255,
 						params->type.text.color2.a);
-	  }
-	else if (ep->part->effect == EDJE_TEXT_EFFECT_SHADOW)
-	  {
+             break;
+           case EDJE_TEXT_EFFECT_SHADOW:
 	     style = EVAS_TEXT_STYLE_SHADOW;
 	     evas_object_text_shadow_color_set(ep->object,
 					       (params->type.text.color3.r * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.g * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.b * params->type.text.color3.a) / 255,
 					       params->type.text.color3.a);
-	  }
-	else if (ep->part->effect == EDJE_TEXT_EFFECT_SOFT_SHADOW)
-	  {
+             break;
+           case EDJE_TEXT_EFFECT_SOFT_SHADOW:
 	     style = EVAS_TEXT_STYLE_SOFT_SHADOW;
 	     evas_object_text_shadow_color_set(ep->object,
 					       (params->type.text.color3.r * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.g * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.b * params->type.text.color3.a) / 255,
 					       params->type.text.color3.a);
-	  }
-	else if (ep->part->effect == EDJE_TEXT_EFFECT_OUTLINE_SHADOW)
-	  {
+             break;
+           case EDJE_TEXT_EFFECT_OUTLINE_SHADOW:
 	     style = EVAS_TEXT_STYLE_OUTLINE_SHADOW;
 	     evas_object_text_outline_color_set(ep->object,
 						(params->type.text.color2.r * params->type.text.color2.a) / 255,
@@ -647,9 +676,8 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 					       (params->type.text.color3.g * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.b * params->type.text.color3.a) / 255,
 					       params->type.text.color3.a);
-	  }
-	else if (ep->part->effect == EDJE_TEXT_EFFECT_OUTLINE_SOFT_SHADOW)
-	  {
+             break;
+           case EDJE_TEXT_EFFECT_OUTLINE_SOFT_SHADOW:
 	     style = EVAS_TEXT_STYLE_OUTLINE_SOFT_SHADOW;
 	     evas_object_text_outline_color_set(ep->object,
 						(params->type.text.color2.r * params->type.text.color2.a) / 255,
@@ -661,48 +689,84 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 					       (params->type.text.color3.g * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.b * params->type.text.color3.a) / 255,
 					       params->type.text.color3.a);
-	  }
-	else if (ep->part->effect == EDJE_TEXT_EFFECT_FAR_SHADOW)
-	  {
+             break;
+           case EDJE_TEXT_EFFECT_FAR_SHADOW:
 	     style = EVAS_TEXT_STYLE_FAR_SHADOW;
 	     evas_object_text_shadow_color_set(ep->object,
 					       (params->type.text.color3.r * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.g * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.b * params->type.text.color3.a) / 255,
 					       params->type.text.color3.a);
-	  }
-	else if (ep->part->effect == EDJE_TEXT_EFFECT_FAR_SOFT_SHADOW)
-	  {
+             break;
+           case EDJE_TEXT_EFFECT_FAR_SOFT_SHADOW:
 	     style = EVAS_TEXT_STYLE_FAR_SOFT_SHADOW;
 	     evas_object_text_shadow_color_set(ep->object,
 					       (params->type.text.color3.r * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.g * params->type.text.color3.a) / 255,
 					       (params->type.text.color3.b * params->type.text.color3.a) / 255,
 					       params->type.text.color3.a);
-	  }
-	else if (ep->part->effect == EDJE_TEXT_EFFECT_GLOW)
-	  {
+             break;
+           case EDJE_TEXT_EFFECT_GLOW:
 	     style = EVAS_TEXT_STYLE_GLOW;
 	     evas_object_text_glow_color_set(ep->object,
-						(params->type.text.color2.r * params->type.text.color2.a) / 255,
-						(params->type.text.color2.g * params->type.text.color2.a) / 255,
-						(params->type.text.color2.b * params->type.text.color2.a) / 255,
-						params->type.text.color2.a);
+                                             (params->type.text.color2.r * params->type.text.color2.a) / 255,
+                                             (params->type.text.color2.g * params->type.text.color2.a) / 255,
+                                             (params->type.text.color2.b * params->type.text.color2.a) / 255,
+                                             params->type.text.color2.a);
 	     evas_object_text_glow2_color_set(ep->object,
-					       (params->type.text.color3.r * params->type.text.color3.a) / 255,
-					       (params->type.text.color3.g * params->type.text.color3.a) / 255,
-					       (params->type.text.color3.b * params->type.text.color3.a) / 255,
-					       params->type.text.color3.a);
-	  }
+                                              (params->type.text.color3.r * params->type.text.color3.a) / 255,
+                                              (params->type.text.color3.g * params->type.text.color3.a) / 255,
+                                              (params->type.text.color3.b * params->type.text.color3.a) / 255,
+                                              params->type.text.color3.a);
+             break;
+           default:
+	     style = EVAS_TEXT_STYLE_PLAIN;
+             break;
+          }
+        
+        switch (effect & EDJE_TEXT_EFFECT_MASK_SHADOW_DIRECTION)
+          {
+           case EDJE_TEXT_EFFECT_SHADOW_DIRECTION_BOTTOM_RIGHT:
+             EVAS_TEXT_STYLE_SHADOW_DIRECTION_SET
+                (style, EVAS_TEXT_STYLE_SHADOW_DIRECTION_BOTTOM_RIGHT);
+             break;
+           case EDJE_TEXT_EFFECT_SHADOW_DIRECTION_BOTTOM:
+             EVAS_TEXT_STYLE_SHADOW_DIRECTION_SET
+                (style, EVAS_TEXT_STYLE_SHADOW_DIRECTION_BOTTOM);
+             break;
+           case EDJE_TEXT_EFFECT_SHADOW_DIRECTION_BOTTOM_LEFT:
+             EVAS_TEXT_STYLE_SHADOW_DIRECTION_SET
+                (style, EVAS_TEXT_STYLE_SHADOW_DIRECTION_BOTTOM_LEFT);
+             break;
+           case EDJE_TEXT_EFFECT_SHADOW_DIRECTION_LEFT:
+             EVAS_TEXT_STYLE_SHADOW_DIRECTION_SET
+                (style, EVAS_TEXT_STYLE_SHADOW_DIRECTION_LEFT);
+             break;
+           case EDJE_TEXT_EFFECT_SHADOW_DIRECTION_TOP_LEFT:
+             EVAS_TEXT_STYLE_SHADOW_DIRECTION_SET
+                (style, EVAS_TEXT_STYLE_SHADOW_DIRECTION_TOP_LEFT);
+             break;
+           case EDJE_TEXT_EFFECT_SHADOW_DIRECTION_TOP:
+             EVAS_TEXT_STYLE_SHADOW_DIRECTION_SET
+                (style, EVAS_TEXT_STYLE_SHADOW_DIRECTION_TOP);
+             break;
+           case EDJE_TEXT_EFFECT_SHADOW_DIRECTION_TOP_RIGHT:
+             EVAS_TEXT_STYLE_SHADOW_DIRECTION_SET
+                (style, EVAS_TEXT_STYLE_SHADOW_DIRECTION_TOP_RIGHT);
+             break;
+           case EDJE_TEXT_EFFECT_SHADOW_DIRECTION_RIGHT:
+             EVAS_TEXT_STYLE_SHADOW_DIRECTION_SET
+                (style, EVAS_TEXT_STYLE_SHADOW_DIRECTION_RIGHT);
+             break;
+           default:
+             break;
+          }
 	evas_object_text_style_set(ep->object, style);
      }
-
-   if (free_text)
-     free((char *)text);
-   if (font2)
-     free(font2);
-   if (sfont)
-     free(sfont);
+   
+   if (free_text) free((char *)text);
+   if (font2) free(font2);
+   if (sfont) free(sfont);
 }
 
 Evas_Font_Size
